# autogenerated
from typing import Literal

LayerTitle = Literal[
    "Кадастровые округа",
    "Территории объектов культурного наследия",
    "Кадастровые районы ",
    "Кадастровые кварталы",
    "Государственная граница Российской Федерации",
    "Субъекты Российской Федерации (линии)",
    "Субъекты Российской Федерации (полигоны)",
    "Муниципальные образования (полигональный)",
    "Муниципальные образования (линейный)",
    "Населённые пункты (полигоны)",
    "Населённые пункты (линии)",
    "Земельные участки из ЕГРН",
    "Земельные участки, образуемые по проекту межевания территории (линейный)",
    "Земельные участки, образуемые по проекту межевания территории (полигональный)",
    "Земельные участки, образуемые по схеме расположения земельного участка",
    "Земельные участки, выставленные на аукцион ",
    "Земельные участки, свободные от прав третьих лиц",
    "Земля для стройки ПКК",
    "Земля для туризма ПКК",
    "Здания",
    "Сооружения",
    "Объекты незавершенного строительства",
    "Единые недвижимые комплексы",
    "Предприятие как имущественный комплекс",
    "ЗОУИТ объектов культурного наследия",
    "ЗОУИТ объектов энергетики, связи, транспорта",
    "ЗОУИТ природных территорий",
    "ЗОУИТ охраняемых объектов и безопасности",
    "Иные ЗОУИТ",
    "Территориальные зоны",
    "Красные линии ",
    "Особо охраняемые природные территории ",
    "Охотничьи угодья ",
    "Лесничества",
    "Береговые линии (границы водных объектов) (полигональный)",
    "Береговые линии (границы водных объектов)(линейный)",
    "Особые экономические зоны",
    "Территории опережающего развития",
    "Зоны территориального развития в Российской Федерации",
    "Игорные зоны",
    "Территории выполнения комплексных кадастровых работ",
    "Территория проведения мероприятий по ликвидации накопленного вреда окружающей среде, образовавшегося в результате производства химической продукции в г. Усолье-Сибирское Иркутской области",
    "Негативные процессы",
    "Объекты туристского интереса",
    "Кадастровая стоимость объекта",
    "Удельный показатель кадастровой стоимости",
]
